unit Hurst1;

interface
uses stat2, math, dialogs, sysutils;


function HurstEst1(var Data: array of double; n: integer; p, q: Double;
                   var H1, S1: double; var Weight: Double; var RepNum: Integer): Boolean;

implementation


function HurstEst1;
//(var Data: array of double; n: integer; p, q: Double; var H1, S1, Weight: Double; RepNum: Integer): Boolean;
{New routine for the estimation of Hurst coefficient, based on Koutsoyiannis, 2002}
const
  toler = 0.00005;
  RepMax = 50;
  MinWeight = 0.001;

var
//  Weight: Double;
//  RepNum: Integer;
  kmax, k: Integer;
  S0, Sk, Sc, Ss: Double;
  SS0, SSk, SSc, SSs: Double;
  Num1, Num2, Denom: Double;
  H0: Double;
  V0, Vk, Vc, Vs, Vln: array of double;

  function AggrStDev(var a: array of double; n: integer; sscale: Integer): Double;
  var
    i, j, count: Integer;
    sum, sumsq: Double;
    aa: array of double;
  begin
    count:=n div sscale;
    setlength (aa, count);
    For i:=0 to count-1 do
    begin
      sum:=0;
      For j:= i*sscale to (i+1)*sscale-1 do sum:=sum+a[j];
      aa[i]:=sum;
    end;
    sum:=0;
    For i:=0 to count-1 do sum:=sum+aa[i];
    sum:=sum/count;
    sumsq:=0;
    For i:=0 to count-1 do sumsq:=sumsq+SQR(aa[i]-sum);
    Result:=SQRT(sumsq/(count-1));
    aa := nil;
  end; {function AggrStDev}

  function SumArray(var a: array of double; n: integer): Double;
  var
    i: Integer;
    sum: Double;
  begin
    sum:=0;
    For i:=0 to n-1 do sum:=sum+a[i];
    Result:=sum;
  end; {function SumArray}

  function SumProduct(var a, b: array of double; n: integer): Double;
  var
    i: Integer;
    sum: Double;
  begin
    sum:=0;
    For i:=0 to n-1 do sum:=sum+a[i]*b[i];
    Result:=sum;
  end; {function SumProduct}

  function cc(n, k: Integer; H: Double): Double;
  begin
    cc:=SQRT((n/k-Power(n/k, 2*H-1))/(n/k-0.5));
  end; {function cc}

  function dd(n, k: Integer; H: Double): Double;
  begin
    dd:=Ln(n/k)/(1-Power(n/k, 2-2*H));
  end; {function dd}

begin
//  showmessage (format('%d %f %f', [n, data[0], data[n-1]]));
  kmax:=n div 10;

  setlength (V0, kmax);
  setlength (Vk, kmax);
  setlength (Vc, kmax);
  setlength (Vs, kmax);
  setlength (Vln, kmax);

  For k:=1 to kmax do
  begin
    V0[k-1]:=1/Power(k, p);
    Vk[k-1]:=Ln(k)/Power(k, p);
    Vs[k-1]:=Ln(AggrStDev(Data, n, k))/Power(k, p);
  end;
  S0:=SumArray(V0, kmax);
  Sk:=SumArray(Vk, kmax);
  Ss:=SumArray(Vs, kmax);

  RepNum:=0;
  Weight:=1;
  H1:=0.50;

  Repeat
    RepNum:=RepNum+1;
    H0:=H1;
    For k:=1 to kmax do
    begin
      Vc[k-1]:=Ln(cc(n, k, H0))/Power(k, p);
      Vln[k-1]:=Ln(k)+dd(n, k, H0);
    end;
    Sc:=SumArray(Vc, kmax);
    SS0:=SumProduct(V0, Vln, kmax);
    SSk:=SumProduct(Vk, Vln, kmax);
    SSs:=SumProduct(Vs, Vln, kmax);
    SSc:=SumProduct(Vc, Vln, kmax);
    SSs:=SSs-Power(H0, q);
    Num1:=SSs*S0-Ss*SS0;
    Num2:=SSc*S0-Sc*SS0;
    Denom:=SSk*S0-Sk*SS0;
    H1:=(Num1/Denom-Num2/Denom)*Weight+H0*(1-Weight);
    If H1>1 then H1:=1-(1-H0)*0.8;
    If H1<0 then H1:=H0*0.8;
    If RepNum=RepMax then
    begin
      Weight:=Weight/2;
      RepNum:=0;
    end;
  until (ABS(H1-H0)<toler) or (Weight<MinWeight);

  S1:=H1*Sk+Sc-Ss;
  S1:=exp(-S1/S0);

  Result := Weight >= MinWeight;

  V0 := nil; Vk:= nil; Vc:= nil; Vs:= nil; Vln:= nil;

end; {function Hurst1}


end.
